<?php 

/**
 Pluginbuilder Controller
 
 @Copyright 2016 Stefan Rick
 @author Stefan Rick
 Mail: stefan@rick-software.de
 Web: http://www.netzberater.de

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


class Pluginbuilder extends Service {
		
	public $scriptPath = '/opt/max2play/';		
	
	public function __construct(){
		parent::__construct();
		
		if($_REQUEST['ajax'] == 1 && $_REQUEST['action'] == 'getcustomfiles'){
			ob_end_clean();
			echo json_encode($this->_getCustomFilesForPlugin($_REQUEST['pluginname']));
			ob_flush();
			die();
		}
		
		$this->getVersion();
		if($this->info->version < '2.34'){
			$this->view->message[] = str_replace(array('NEEDEDVERSION', 'YOURVERSION'),array('2.34', $this->info->version),_t('You need at least Max2Play Version NEEDEDVERSION. Your Max2Play Version is YOURVERSION. Please update (on settings page) before using this addon.'));
		}
		
		//Set your Pluginname
		$this->pluginname = _('Pluginbuilder');		
		$this->registerLocale(dirname(__FILE__).'/../locale', 'pluginbuilder');
		
		//If Button clicked or Form sent do Something
		if(isset($_REQUEST['action'])){
			if($_REQUEST['action'] == 'createPlugin'){
				$this->_createPlugin();
			}
			if($_REQUEST['action'] == 'delete'){
				$this->_delete();
			}
			if($_REQUEST['action'] == 'download'){
				$this->_download();
			}			
		}
		$this->_getCreatedPlugins();		
		//Get Debug Info
		$this->_getAllLogs();
		
		$this->showHelpSidebar();
	}
	
	private function _download(){		
		$plugin = $_REQUEST['createdPlugins'];
		$filename = '/var/www/max2play/public/cache/'.$plugin.'.tar';
		$script = array();
		
		//check for tar Folder with access rights
		if(!file_exists('/var/www/max2play/public/cache')){
			$this->writeDynamicScript(array('mkdir /var/www/max2play/public/cache; chmod 777 /var/www/max2play/public/cache;'));
			$this->view->message[] = _t('Cachefolder created');
		}
		// remove existing files
		if(file_exists($filename)){
			$script[] = 'rm '.$filename;			
		}
		$script[] = "cd /var/www/max2play/application/plugins && tar -cvf $filename $plugin";
		$this->writeDynamicScript($script);

		//$phar = new PharData($filename);
		// add all files in the project
		//$phar->buildFromDirectory('/var/www/max2play/application/plugins/', '/'.$plugin.'/');
		$this->view->message[] = _t('Generated .tar File from Plugin: <a href="/cache/'.$plugin.'.tar">Download '.$plugin.'.tar</a>');
		return true;
	}
	
	private function _delete(){		
		$plugin = $_REQUEST['createdPlugins'];
		$this->removePlugins(array($plugin));
		$this->view->message[] = nl2br($this->writeDynamicScript(array('rm -R /var/www/max2play/application/plugins/'.$plugin)));
		$this->loadViewHeader(true);
		$this->view->message[] = _t('Deleted Plugin').' '.$plugin;
		return true;
	}
	
	/**
	 * Get all custom Plugins from Max2Play-Folder
	 * Read by Plugin Setup file -> custom Plugins created by this addon have a Status flag 
	 */
	private function _getCreatedPlugins(){
		$this->parsePlugins();
		$this->view->customplugins = array();
		foreach($this->view->pluginselect as $plugin){
			if($plugin['custom'])
				$this->view->customplugins[] = $plugin;
			elseif($this->debug == true)
				$this->view->customplugins[] = $plugin;
		}
		return true;
	}
	
	/**
	 * Demo function to do Something
	 * e.g. Call Scripts and return Output to view
	 */
	private function _createPlugin(){		
		$pluginname = ucfirst(preg_replace('=[^a-zA-Z_0-9]=','',$_REQUEST['pluginname']));		
		
		if(strlen($pluginname) > 3){
			$this->view->message[] = _('Creating Plugin '.$pluginname);				
					
			$output = nl2br($this->writeDynamicScript(array('echo "'.$pluginname.'" | /opt/max2play/createplugin.sh')));
			
			$this->view->message[] = $output;
			
			//Reload Plugin Config and Enable new Plugin		
			$this->enablePlugin($pluginname);
			$this->loadViewHeader(true);
			$this->view->message[] = _t('Plugin created and activated - you may now edit its content, test it, and share it with the community!');
		}else{
			$this->view->message[] = _t('Pluginname is not correct: '.$pluginname);
		}			
		
		return true;
	}		
	
	private function _getAllLogs(){		
		$out = array();
		$this->view->debug = $out;
	}
	
	/**
	 * Ajax Function to show all Custom Files for Plugin to Edit
	 */
	private function _getCustomFilesForPlugin($pluginname = ''){
		$files = array();
		$dir = APPLICATION_PATH.'/plugins/'.$pluginname;
		if(file_exists($dir)){
			// get all Files within Directory scripts
			$handle_controller = opendir($dir);
			if($handle_controller){
				while (false !== ($file = readdir($handle_controller))) {
					if ($file != "." && $file != "..") {
						if(is_dir($dir.'/'.$file)){
							// Second Directory
							$handle2_controller = opendir($dir.'/'.$file);
							if($handle2_controller){
								while (false !== ($file2 = readdir($handle2_controller))) {
									if ($file2 != "." && $file2 != "..") {
										if(is_dir($dir.'/'.$file.'/'.$file2)){
											// third... Directory											
										}else
											$files[] = array('value' => $file.'/'.$file2, 'text' => $file.'/'.$file2);
									}
								}
							}
						}else
							$files[] = array('value' => $file, 'text' => $file);
					}
				}
			}
		}
		return $files;
	}
	
	public function showHelpSidebar(){
		global $helpSidebar;
		$helpSidebar['title'] = _t('Help - Pluginbuilder');
		$helpSidebar['content'] = _t('<ul><li>Build your Plugin with Max2Play - Find HowTos in our developer corner.</li><li>A plugin basically consists of a controller file (application logic) and a view file (HTML generation).</li></ul>');
		return true;
	}
}

$builder = new Pluginbuilder();

include_once(dirname(__FILE__).'/../view/setup.php');

